import "package:moxxyv2/ui/pages/conversation/conversation.dart";
import "package:moxxyv2/ui/pages/conversations.dart";
import "package:moxxyv2/ui/pages/profile/profile.dart";
import "package:moxxyv2/ui/pages/newconversation.dart";
import "package:moxxyv2/ui/pages/login/login.dart";
import "package:moxxyv2/ui/pages/register/register.dart";
import "package:moxxyv2/ui/pages/intro.dart";
import "package:moxxyv2/ui/pages/addcontact/addcontact.dart";
import "package:moxxyv2/ui/pages/postregister/postregister.dart";
import "package:moxxyv2/ui/pages/sendfiles.dart";
import "package:moxxyv2/ui/pages/settings/settings.dart";
import "package:moxxyv2/ui/pages/settings/licenses.dart";
import "package:moxxyv2/ui/pages/settings/about.dart";
import "package:moxxyv2/ui/constants.dart";
import "package:moxxyv2/redux/conversation/actions.dart";
import "package:moxxyv2/redux/conversations/middlewares.dart";
import "package:moxxyv2/redux/account/middlewares.dart";
import "package:moxxyv2/redux/account/actions.dart";
import "package:moxxyv2/redux/login/middlewares.dart";
import "package:moxxyv2/redux/login/actions.dart";
import "package:moxxyv2/redux/registration/middlewares.dart";
import "package:moxxyv2/redux/addcontact/middlewares.dart";
import "package:moxxyv2/redux/addcontact/actions.dart";
import "package:moxxyv2/redux/roster/middlewares.dart";
import "package:moxxyv2/redux/roster/actions.dart";
import "package:moxxyv2/redux/messages/middleware.dart";
import "package:moxxyv2/redux/conversation/middlewares.dart";
import "package:moxxyv2/redux/state.dart";
import "package:moxxyv2/redux/start/middlewares.dart";
import "package:moxxyv2/models/conversation.dart";
import "package:moxxyv2/models/message.dart";
import "package:moxxyv2/models/roster.dart";
import "package:moxxyv2/backend/account.dart";
import "package:moxxyv2/service/xmpp.dart";

import "package:flutter/material.dart";
import "package:flutter/foundation.dart";
import "package:get_it/get_it.dart";
import "package:flutter_redux/flutter_redux.dart";
import "package:flutter_redux_navigation/flutter_redux_navigation.dart";
import "package:redux_logging/redux_logging.dart";
import "package:redux/redux.dart";
import "package:flutter_background_service/flutter_background_service.dart";

Store<MoxxyState> createStore() {
  final store = Store<MoxxyState>(
    moxxyReducer,
    initialState: MoxxyState.initialState(),
    middleware: [
      conversationsMiddleware,
      startMiddleware,
      accountMiddleware,
      loginMiddleware,
      registrationMiddleware,
      addcontactMiddleware,
      rosterMiddleware,
      messageMiddleware,
      conversationMiddleware,
      const NavigationMiddleware(),

      // We only need this while debugging
      ...(kDebugMode ? [ LoggingMiddleware.printer() ] : [])
    ]
  );
  
  return store;
}

// TODO: Replace all Column(children: [ Padding(), Padding, ...]) with a
//       Padding(padding: ..., child: Column(children: [ ... ]))
// TODO: Theme the switches
// TODO: Find a better way to do this
void main() async {
  final store = createStore();

  await initializeServiceIfNeeded();
  
  GetIt.I.get<FlutterBackgroundService>().onDataReceived.listen((data) {
      if (data!["type"]! != "__LOG__") {
        // TODO: Use logging function and only print on when debugging
        // ignore: avoid_print
        print("GOT: " + data.toString());
      }

      switch (data["type"]) {
        case "PreStartResult": {
          if (data["state"] == "logged_in") {
            FlutterBackgroundService().sendData({
                "type": "LoadConversationsAction"
            });
            /* TODO: Move this into the XmppRepository
            FlutterBackgroundService().sendData({
                "type": "GetAccountStateAction"
            });
            */

            (() async {
                final state = await getAccountData();
                store.dispatch(SetAccountAction(state: state!));
            })();
            
            store.dispatch(NavigateToAction.replace("/conversations"));
          } else {
            store.dispatch(NavigateToAction.replace("/intro"));
          }
        }
        break;
        case "LoginSuccessfulEvent": {
          store.dispatch(
            LoginSuccessfulAction(
              jid: data["jid"]!,
              displayName: data["displayName"]!
            )
          );
        }
        break;
        case "ConversationCreatedEvent": {
          store.dispatch(AddConversationAction(
              conversation: Conversation.fromJson(data["conversation"]!)
            )
          );
        }
        break;
        case "ConversationUpdatedEvent": {
          store.dispatch(
            UpdateConversationAction(
              conversation: Conversation.fromJson(data["conversation"]!)
            )
          );
        }
        break;
        case "MessageReceivedEvent": {
          store.dispatch(
            AddMessageAction(
              message: Message.fromJson(data["message"]!)
            )
          );
        }
        break;
        case "LoadRosterItemsResult": {
          final List<RosterItem> tmp = List<RosterItem>.from(data["items"]!.map((i) => RosterItem.fromJson(i)));
          store.dispatch(
            AddMultipleRosterItemsAction(
              items: tmp
            )
          );
        }
        break;
        case "LoadConversationsResult": {
          final List<Conversation> tmp = List<Conversation>.from(data["conversations"]!.map((c) => Conversation.fromJson(c)));
          store.dispatch(AddMultipleConversationsAction(
              conversations: tmp
          ));
        }
        break;
        case "LoadMessagesForJidResult": {
          final List<Message> tmp = List<Message>.from(data["messages"]!.map((m) => Message.fromJson(m)));
          store.dispatch(
            AddMultipleMessagesAction(
              conversationJid: data["jid"]!,
              messages: tmp,
              replace: true
            )
          );
        }
        break;
        case "AddToRosterResult": {
          store.dispatch(
            AddToRosterDoneAction(
              result: data["result"]!,
              msg: data["msg"],
              jid: data["jid"]
            )
          );
        }
        break;
        case "RosterItemModifiedEvent": {
          store.dispatch(
            ModifyRosterItemAction(
              item: RosterItem.fromJson(data["item"]!)
            )
          );
        }
        break;
        case "MessageSendResult": {
          store.dispatch(
            AddMessageAction(
              message: Message.fromJson(data["message"]!)
            )
          );
        }
        break;
        case "__LOG__": {
          // TODO: Use logging function and only print on when debugging
          // ignore: avoid_print
          print("[S] " + data["log"]!);
        }
        break;
      }
  });
  
  runApp(MyApp(store: store));
}

// TODO: Move somewhere else
class SplashScreen extends StatelessWidget {
  const SplashScreen({ Key? key }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Center(
        child: Image.asset(
          "assets/images/logo.png",
          width: 200, height: 200
        )
      )
    );
  }
}

class MyApp extends StatefulWidget {
  final Store<MoxxyState> store;

  const MyApp({Key? key, required this.store }) : super(key: key);

  @override
  // ignore: no_logic_in_create_state
  _MyAppState createState() => _MyAppState(store: store);
}

class _MyAppState extends State<MyApp> with WidgetsBindingObserver {
  final Store<MoxxyState> store;

  _MyAppState({ required this.store });

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance!.addObserver(this);
  }

  @override
  void dispose() {
    WidgetsBinding.instance!.removeObserver(this);
    super.dispose();
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    super.didChangeAppLifecycleState(state);

    switch (state) {
      case AppLifecycleState.paused:
        FlutterBackgroundService().sendData({
            "type": "SetCSIState",
            "state": "background"
        });
        break;
      case AppLifecycleState.resumed:
        FlutterBackgroundService().sendData({
            "type": "SetCSIState",
            "state": "foreground"
        });
        break;
      default: break;
    }
  }
  
  @override
  Widget build(BuildContext context) {
    return StoreProvider(
      store: store,
      child: MaterialApp(
        title: "Moxxy",
        theme: ThemeData(
          primarySwatch: Colors.blue,
        ),
        darkTheme: ThemeData(
          brightness: Brightness.dark,
          elevatedButtonTheme: ElevatedButtonThemeData(
            style: ElevatedButton.styleFrom(
              primary: primaryColor,
              onPrimary: Colors.white
            )
          ),
          textButtonTheme: TextButtonThemeData(
            style: TextButton.styleFrom(
              primary: primaryColor
            )
          ),
          // NOTE: Mainly for the SettingsSection
          colorScheme: const ColorScheme.dark(
            secondary: primaryColor
          )
        ),
        navigatorKey: NavigatorHolder.navigatorKey,
        //themeMode: ThemeMode.system,
        themeMode: ThemeMode.dark,
        routes: {
          "/intro": (context) => const IntroPage(),
          "/login": (context) => LoginPage(),
          "/register": (context) => RegistrationPage(),
          "/register/post": (context) => const PostRegistrationPage(),
          "/conversations": (context) => const ConversationsPage(),
          "/conversation": (context) => const ConversationPage(),
          "/conversation/profile": (context) => const ProfilePage(),
          "/conversation/send_files": (context) => const SendFilesPage(),
          "/new_conversation": (context) => const NewConversationPage(),
          "/new_conversation/add_contact": (context) => AddContactPage(),
          "/settings": (context) => const SettingsPage(),
          "/settings/licenses": (context) => const SettingsLicensesPage(),
          "/settings/about": (context) => const SettingsAboutPage(),
        },
        home: const SplashScreen(),
      )
    );
  }
}
